@@ -58,6 +58,16 @@
 #  define MY_END_ALLOW_THREADS(st)      { st = NULL; }
 #endif
 
+/* Workaround for mingw32 + Python 2.4: if we don't do this, mystrdup is used
+ * from msvcrt60.dll and the buffer is then free()-ed from msvcr71.dll, which
+ * results in a crash.
+ */
+#if defined(__MINGW32__)
+#define mystrdup _strdup
+#else
+#define mystrdup strdup
+#endif
+
 /*------------------------------------------------------------------------------
 ** Object Declarations
 **------------------------------------------------------------------------------
@@ -262,7 +272,7 @@ PyObject* pysqlite_connect(PyObject *self, PyObject *args, PyObject *kwargs)
 
 
     /* Assign the database name */
-    if ((obj->database_name = strdup(db_name)) == NULL)
+    if ((obj->database_name = mystrdup(db_name)) == NULL)
     {
         PyErr_SetString(PyExc_MemoryError, "Cannot allocate memory for database name.");
         return NULL;
@@ -1063,7 +1073,7 @@ static int my_sqlite3_exec(
     {
         db = con->p_db;
         MY_BEGIN_ALLOW_THREADS(con->tstate)
-        rc = sqlite3_prepare(db, tail, 0, &statement, &tail);
+        rc = sqlite3_prepare(db, tail, -1, &statement, &tail);
         MY_END_ALLOW_THREADS(con->tstate)
 
         if (rc != SQLITE_OK)
@@ -1180,9 +1190,6 @@ static int my_sqlite3_exec(
                 if (rc == SQLITE_DONE)
                     break;
             }
-
-            free(p_fields);
-            free(p_col_names);
         }
         else if (rc != SQLITE_BUSY)
         {
@@ -1221,6 +1228,10 @@ static int my_sqlite3_exec(
                 PyTuple_SetItem(p_rset->p_col_def_list, i, p_col_def);
             }
         }
+
+        free(p_fields);
+        free(p_col_names);
+
         rc = sqlite3_finalize(statement);
 
         /* Because it worked for older PySQLite versions, we allow here to
@@ -1282,7 +1293,7 @@ static PyObject* _con_execute(pysqlc* self, PyObject *args)
     }
 
     /* Save SQL statement */
-    self->sql = strdup(sql);
+    self->sql = mystrdup(sql);
 
     /* Log SQL statement */
     if (self->command_logfile != Py_None)
@@ -1334,7 +1345,7 @@ static PyObject* _con_execute(pysqlc* self, PyObject *args)
             return NULL;
         }
 
-        if ((buf = strdup(sql)) == NULL)
+        if ((buf = mystrdup(sql)) == NULL)
         {
             PyErr_SetString(PyExc_MemoryError, "Cannot allocate buffer for copying SQL statement!");
             return NULL;
@@ -1480,15 +1491,15 @@ int process_record(sqlite3_stmt* statement, void* p_data, int num_fields, char**
             {
                 type_code = tc_INTEGER;
             }
+            else if (strstr(type_name, "UNICODE"))
+            {
+                type_code = tc_UNICODESTRING;
+            }
             else if (strstr(type_name, "CHAR")
                   || strstr(type_name, "TEXT"))
             {
                 type_code = tc_STRING;
             }
-            else if (strstr(type_name, "UNICODE"))
-            {
-                type_code = tc_UNICODESTRING;
-            }
             else if (strstr(type_name, "BINARY")
                   || strstr(type_name, "BLOB"))
             {
@@ -1841,7 +1852,7 @@ static PyObject* sqlite_version_info(PyObject* self, PyObject* args)
         return NULL;
     }
 
-    buf = strdup(sqlite3_libversion());
+    buf = mystrdup(sqlite3_libversion());
     iterator = buf;
 
     vi_list = PyList_New(0);
